--[[
Name: DebugComponent-1.0
Revision: $Rev: 41 $
Author(s): sylvanaar (sylvanaar@mindspring.com)
Documentation: http://www.wowace.com/wiki/Debug-1.0
SVN: http://svn.wowace.com/root/trunk/Debug-1.0/DebugComponent-1.0
Description: An AceDebug extension to support subsystem logging
Dependencies: AceLibrary, Ace-00
License: LGPL v2.1
]]


local MAJOR_VERSION = "DebugComponent-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 41 $"):match("(%d+)"))

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0") end 



--
-- Usage:
--
-- Requires AceDebug
--
--  YourMod = AceLibrary("AceAddon-2.0"):new( ..., "AceDebug-2.0", "Debug-1.0")
--
-- in OnInitialize set the initial state of each subsystem
--
--    self:SetDebugComponent("ORLY", true);
--    self:SetDebugComponent("NOWAI", true)
--
-- typically its best to make helper functions:
--
--   local function LOG_ORLY(...) YourMod:DebugComponent("ORLY", ...) end
--   local function LOG_NOWAI(...) YourMod:DebugComponent("NOWAI", ...) end
-- 
--   The menu will apear when you enable debugging, and 
--   let you toggle the subsystems on and off



local AceOO = AceLibrary:GetInstance("AceOO-2.0")
local DBG = AceOO.Mixin {
	"SetDebugComponent",
	"GetDebugComponent",
	
	"DebugComponent",

	"DumpComponent",
	"DumpExComponent",
	
	"GetAceOptionsDataTable",
}

local options

-- #NODOC
local function safecall(func,...)
	local success, err = pcall(func,...)
	if not success then geterrorhandler()(err:find("%.lua:%d+:") and err or (debugstack():match("\n(.-: )in.-\n") or "") .. err) end
end

local clrcache = {}
-- #NODOC
local function GetAutoGenColorCLR(Name)
    if Name and clrcache[Name] then 
        return clrcache[Name]
    end
    
    if not Name then return "ffffff" end
        
	local hash = 17
	for i=1, Name:len() do
		hash = hash * 37 * Name:byte(i);
	end

	local r = math.floor(math.fmod(hash / 97, 255));
	local g = math.floor(math.fmod(hash / 17, 255));
	local b = math.floor(math.fmod(hash / 227, 255));

    if ((r * 299 + g * 587 + b * 114) / 1000) < 105 then
    	r = math.abs(r - 255);
        g = math.abs(g - 255);
        b = math.abs(b - 255);
    end

    clrcache[Name] = ("%02x%02x%02x"):format(r, g, b)
	return clrcache[Name]
end

function DBG:SetDebugComponent(component, value)
    local d = self.dbglib
    
    d.cmpts = d.cmpts or {}
    
    if not value then 
        if self:IsDebugging() then
            if type(self.OnDebugComponentChange) == "function" then
    			safecall(self.OnDebugComponentChange, self)
    		else
    		    self:DebugComponent(component, value and "|cff80ff80ENABLED|r" or "|cffff8080DISABLED|r")
    		end 
        end        
        d.cmpts[component] = value
    end
    
    if options and options.debugSubsystem then
        local a = options.debugSubsystem.args
        for k,v in pairs(d.cmpts) do
            a[k] = a[k] or { }
            a[k].name, a[k].desc, a[k].type  = k, k, "toggle"
        end
    end
    


    if value then 
        d.cmpts[component] = value

        if self:IsDebugging() then
            if type(self.OnDebugComponentChange) == "function" then
    			safecall(self.OnDebugComponentChange, self)
    		else
    		    self:DebugComponent(component, value and "|cff80ff80ENABLED|r" or "|cffff8080DISABLED|r")
    		end 
        end        
    end
end

function DBG:GetDebugComponent(component)
    return self.dbglib.cmpts[component]
end




function DBG:DebugComponent(component, ...)
    local d = self.dbglib
    if d.cmpts[component] then
        d.dbglibcmpt = component
		if type(self.DebugC) == "function" then
			self.DebugC(self, ...)
		else
		    if self:IsDebugging() then 
    		    self:Debug("<|cff"..GetAutoGenColorCLR(d.dbglibcmpt)..d.dbglibcmpt.."|r> ", ...)
    		else
    		    self:Debug(...)
    		end
		end
		d.dbglibcmpt = nil;
    end
end

function DBG:DumpComponent(component, ...)
    local d = self.dbglib
    
    if d.cmpts[component] then
        d.dbglibcmpt = component
		if type(self.DumpC) == "function" then
			self.DumpC(self, ...)
		else
		    if not self:IsDebugging() then 
    		    self:Debug(nil, nil, nil, ...)
    		end
		end
		d.dbglibcmpt = nil
    end
end

function DBG:DumpExComponent(component, frame, name, order, ...)
    local d = self.dbglib
    if d.cmpts[component] then
        d.dbglibcmpt = component
   		if type(self.DumpExC) == "function" then
			self.DumpExC(self, frame, name, order, ...)
		else
		    if not self:IsDebugging() then 
    		    self:Debug(frame, name, order, ...)
    		end
		end
		d.dbglibcmpt = nil
    end
end


function DBG:GetAceOptionsDataTable(target)
	if not options then		
	    options = {
        debugSubsystem = {
            type = "group",
            order = -2,
            name = "Toggle Subsystem Debugging",
            desc = "Toggle Subsystem Debugging",
            pass = true,
            get  = "GetDebugComponent",
            set  = "SetDebugComponent",
            hidden = "~IsDebugging",
            args = {},
        },			
    }	
	end
	return options
end


function DBG:OnEmbedEnable(addon, first)
    addon.dbglib = addon.dbglib or { }
    addon.dbglib.cmpts  = addon.dbglib.cmpts  or {}
end

-- #NODOC
local function activate(self, oldLib, oldDeactivate)
	DBG = self
end


AceLibrary:Register(DBG, MAJOR_VERSION, MINOR_VERSION, activate)