--[[
Name: DebugStub-1.0
Revision: $Rev: 41 $
Author(s): sylvanaar (sylvanaar@mindspring.com)
Documentation: http://www.wowace.com/wiki/Debug-1.0
SVN: http://svn.wowace.com/root/trunk/Debug-1.0/DebugStub-1.0
Description: A debug print redirector mixin
Dependencies: AceLibrary, Ace-00
License: LGPL v2.1
]]

local MAJOR_VERSION = "DebugStub-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 41 $"):match("(%d+)"))

local AceLibrary = AceLibrary
if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
local AceOO = AceLibrary:GetInstance("AceOO-2.0")


local DBG = AceOO.Mixin {
	"SetDebugFrame",
	"GetDebugFrame",
	
	"SetDebugContext",
	"GetDebugContext",
	
	"Log",
}

-- #NODOC
local function safecall(func,...)
	local success, err = pcall(func,...)
	if not success then geterrorhandler()(err:find("%.lua:%d+:") and 
	    err or (debugstack():match("\n(.-: )in.-\n") or "") .. err) end
end


local _G = getfenv(0)
local _D = _G 


local D_TEXTONLY  = { addon = "", component = null, severity = null, label = null }
local D_SEVERITY  = { addon = "", component = null, severity = 0,    label = null }
local D_COMPONENT = { addon = "", component = null, severity = null, label = null }
local D_ALL       = { addon = "", component = null, severity = null, label = null }

--[[--------------------------------------------------------------------------
--  Debug Context Start (This module will prefix everything though)
------------------------------------------------------------------------]]-----

-- #NODOC
local function print(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
end

-- #NODOC
local function Msg(info, ...)
    if type(info) == "table" then
        local f = _D.DEBUGLIB_DEBUG_FRAME
        
        if f and type(f.Msg) == "function" then    
            safecall(f.Msg, info, ...)
        end
    end
end


--[[----------------------------------------------------------
Arguments:
    string component
    number severity (1-5)
    string label
    ...    The contents of a the debug message

Notes:
    This call bypasses AceDebug and sends directly to the 
    log listener. It also allows you to pass values for
    each field.

------------------------------------------------------------]]
function DBG:Log(addon, component, severity, label, ...)
    D_ALL.addon, D_ALL.component, D_ALL.severity, D.ALL.label = 
        addon, component, severity, label
    Msg(D_ALL, ...)
end


--[[----------------------------------------------------------
Arguments:
	environment - _G, or a custom environment, or nil in which case it uses _G
Notes:
    Since we use a global to locate the reciever this option
    lets you confine the scope by setting up a predefined
    environment which you control the members of.
------------------------------------------------------------]]
function DBG:SetDebugContext(ctx)
    _D = ctx or _G
end


--[[----------------------------------------------------------
Notes:
	Get the current value of _G, or rather, a reference to
	the context where we should look for our event receiever
------------------------------------------------------------]]
function DBG:GetDebugContext()
    return _D
end

--[[----------------------------------------------------------
Notes:
    This allows you to set the reciever directly rather than
    using the shared environment approach. It implicitly 
    sets the environment to 'self'
------------------------------------------------------------]]
function DBG:SetDebugFrame(frame)
    _D = self
    _D.DEBUGLIB_DEBUG_FRAME = frame
end


--[[----------------------------------------------------------
Notes:
    Retrieve the log target listener
------------------------------------------------------------]]
function DBG:GetDebugFrame()
    return _D.DEBUGLIB_DEBUG_FRAME
end

-- #NODOC
local function Debug(addon, ...)
    local d = addon.dbglib
    local t = d.D_TEXTONLY
    if d.dbglibcmpt then 
        d.D_COMPONENT.component = d.dbglibcmpt
        t = d.D_COMPONENT
    end
    
    t.label = d.title
    
    Msg(t, ...)
    
    if addon.super.Debug then 
        addon.super.Debug(addon, ...)
    end
end

-- #NODOC
local function LevelDebug(addon, level, ...)
    local d = addon.dbglib
    d.D_SEVERITY.severity = level

    local t = d.D_SEVERITY
    
    t.label = d.title
    t.component = d.dbglibcmpt

    
    Msg(t, ...)
    
    if addon.super.LevelDebug then 
        addon.super.LevelDebug(addon, level, ...)
    end
end

-- #NODOC
function DBG:OnEmbedInitialize(addon, name)
    addon.Debug = Debug
    addon.LevelDebug = LevelDebug
    
    addon.dbglib = addon.dbglib or { }
    local d = addon.dbglib
    d.D_SEVERITY = { ["addon"] = name }
    d.D_TEXTONLY = { ["addon"] = name }
    d.D_COMPONENT = { ["addon"] = name }
    d.D_ALL = { ["addon"] = name }
    
end

-- #NODOC
function DBG:OnEmbedEnable(addon, first)
    addon.dbglib = addon.dbglib or { }
    addon.dbglib.title = addon.title
    addon.Debug = Debug
    addon.LevelDebug = LevelDebug
end

-- #NODOC
local function activate(self, oldLib, oldDeactivate)
	DBG = self	
end


AceLibrary:Register(DBG, MAJOR_VERSION, MINOR_VERSION, activate)